diff --git a/src/backend/utils/adt/numeric.c b/src/backend/utils/adt/numeric.c
index 6b60a5c..e70230e 100644
*** a/src/backend/utils/adt/numeric.c
--- b/src/backend/utils/adt/numeric.c
***************
*** 30,35 ****
--- 30,37 ----
  #include "catalog/pg_type.h"
  #include "libpq/pqformat.h"
  #include "miscadmin.h"
+ #include "nodes/nodeFuncs.h"
+ #include "parser/parse_clause.h"
  #include "utils/array.h"
  #include "utils/builtins.h"
  #include "utils/int8.h"
***************
*** 713,718 **** numeric_send(PG_FUNCTION_ARGS)
--- 715,762 ----
  
  
  /*
+  * numeric_transform() -
+  *
+  * Flatten calls to our length coercion function that solely represent
+  * increases in allowable precision.  Scale changes mutate every datum, so
+  * they are unoptimizable.  Some values, e.g. 1E-1001, can only fit into an
+  * unconstrained numeric, so a change from an unconstrained numeric to any
+  * constrained numeric is also unoptimizable.
+  */
+ Datum
+ numeric_transform(PG_FUNCTION_ARGS)
+ {
+ 	FuncExpr   *expr = (FuncExpr *) PG_GETARG_POINTER(0);
+ 	Node	   *typmod;
+ 	Node	   *ret = NULL;
+ 
+ 	if (!IsA(expr, FuncExpr))
+ 		PG_RETURN_POINTER(ret);
+ 
+ 	Assert(list_length(expr->args) == 2);
+ 	typmod = lsecond(expr->args);
+ 
+ 	if (IsA(typmod, Const))
+ 	{
+ 		Node	   *source = linitial(expr->args);
+ 		int32		old_typmod = exprTypmod(source);
+ 		int32		new_typmod = DatumGetInt32(((Const *) typmod)->constvalue);
+ 		int32		old_scale = (old_typmod - VARHDRSZ) & 0xffff;
+ 		int32		new_scale = (new_typmod - VARHDRSZ) & 0xffff;
+ 		int32		old_precis = (old_typmod - VARHDRSZ) >> 16 & 0xffff;
+ 		int32		new_precis = (new_typmod - VARHDRSZ) >> 16 & 0xffff;
+ 
+ 		if (new_typmod < VARHDRSZ /* unconstrained destination */ ||
+ 			(old_typmod >= VARHDRSZ /* constrained source */ &&
+ 			 new_scale == old_scale &&
+ 			 new_precis >= old_precis))
+ 			ret = relabel_to_typmod(source, new_typmod);
+ 	}
+ 
+ 	PG_RETURN_POINTER(ret);
+ }
+ 
+ /*
   * numeric() -
   *
   *	This is a special function called by the Postgres database system
diff --git a/src/include/catalog/pg_prindex c893c3a..d0d25b8 100644
*** a/src/include/catalog/pg_proc.h
--- b/src/include/catalog/pg_proc.h
***************
*** 2135,2142 **** DATA(insert OID = 2917 (  numerictypmodin		PGNSP PGUID 12 1 0 0 0 f f f t f i 1
  DESCR("I/O typmod");
  DATA(insert OID = 2918 (  numerictypmodout		PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 2275 "23" _null_ _null_ _null_ _null_	numerictypmodout _null_ _null_ _null_ ));
  DESCR("I/O typmod");
! DATA(insert OID = 1703 ( numeric				PGNSP PGUID 12 1 0 0 0 f f f t f i 2 0 1700 "1700 23" _null_ _null_ _null_ _null_ numeric _null_ _null_ _null_ ));
  DESCR("adjust numeric to typmod precision/scale");
  DATA(insert OID = 1704 ( numeric_abs			PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 1700 "1700" _null_ _null_ _null_ _null_ numeric_abs _null_ _null_ _null_ ));
  DATA(insert OID = 1705 ( abs					PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 1700 "1700" _null_ _null_ _null_ _null_ numeric_abs _null_ _null_ _null_ ));
  DESCR("absolute value");
--- 2135,2144 ----
  DESCR("I/O typmod");
  DATA(insert OID = 2918 (  numerictypmodout		PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 2275 "23" _null_ _null_ _null_ _null_	numerictypmodout _null_ _null_ _null_ ));
  DESCR("I/O typmod");
! DATA(insert OID = 1703 ( numeric				PGNSP PGUID 12 1 0 0 3144 f f f t f i 2 0 1700 "1700 23" _null_ _null_ _null_ _null_ numeric _null_ _null_ _null_ ));
  DESCR("adjust numeric to typmod precision/scale");
+ DATA(insert OID = 3144 ( numeric_transform		PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 2281 "2281" _null_ _null_ _null_ _null_ numeric_transform _null_ _null_ _null_ ));
+ DESCR("transform a numeric length coercion");
  DATA(insert OID = 1704 ( numeric_abs			PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 1700 "1700" _null_ _null_ _null_ _null_ numeric_abs _null_ _null_ _null_ ));
  DATA(insert OID = 1705 ( abs					PGNSP PGUID 12 1 0 0 0 f f f t f i 1 0 1700 "1700" _null_ _null_ _null_ _null_ numeric_abs _null_ _null_ _null_ ));
  DESCR("absolute value");
diff --git a/src/include/utils/builtindex aa36db6..63e2844 100644
*** a/src/include/utils/builtins.h
--- b/src/include/utils/builtins.h
***************
*** 910,915 **** extern Datum numeric_recv(PG_FUNCTION_ARGS);
--- 910,916 ----
  extern Datum numeric_send(PG_FUNCTION_ARGS);
  extern Datum numerictypmodin(PG_FUNCTION_ARGS);
  extern Datum numerictypmodout(PG_FUNCTION_ARGS);
+ extern Datum numeric_transform(PG_FUNCTION_ARGS);
  extern Datum numeric (PG_FUNCTION_ARGS);
  extern Datum numeric_abs(PG_FUNCTION_ARGS);
  extern Datum numeric_uminus(PG_FUNCTION_ARGS);
diff --git a/src/test/regress/expecindex 301cc4b..be669b6 100644
